.\" Copyright (c) 1980, 1991 Regents of the University of California.
.\" All rights reserved.
.\"
.\" Redistribution and use in source and binary forms, with or without
.\" modification, are permitted provided that the following conditions
.\" are met:
.\" 1. Redistributions of source code must retain the above copyright
.\"    notice, this list of conditions and the following disclaimer.
.\" 2. Redistributions in binary form must reproduce the above copyright
.\"    notice, this list of conditions and the following disclaimer in the
.\"    documentation and/or other materials provided with the distribution.
.\" 3. All advertising materials mentioning features or use of this software
.\"    must display the following acknowledgement:
.\"	This product includes software developed by the University of
.\"	California, Berkeley and its contributors.
.\" 4. Neither the name of the University nor the names of its contributors
.\"    may be used to endorse or promote products derived from this software
.\"    without specific prior written permission.
.\"
.\" THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
.\" ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
.\" IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
.\" ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
.\" FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
.\" DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
.\" OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
.\" HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
.\" LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
.\" OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
.\" SUCH DAMAGE.
.\"
.\"     @(#)wait.2	6.5 (Berkeley) 3/10/91
.\"
.TH WAIT 2 "March 12, 1993"
.UC 4
.SH NAME
wait, waitpid, wait4, wait3 \- wait for process terminatation
.SH SYNOPSIS
.ft B
.nf
#include <sys/types.h>
#include <sys/wait.h>
.PP
.ft B
pid = wait(status)
int pid;
union wait *status;
.PP
.ft B
#include <sys/time.h>
#include <sys/resource.h>
.PP
.ft B
pid = waitpid(wpid, status, options);
int pid;
int wpid;
union wait *status;
int options;
.PP
.ft B
pid = wait3(status, options, rusage);
int pid;
union wait *status;
int options;
struct rusage *rusage;
.PP
.ft B
pid = wait4(wpid, status, options, rusage);
int pid;
int wpid;
union wait *status;
int options;
struct rusage *rusage;
.SH DESCRIPTION
The
.I wait
function suspends execution of its calling process until
.B status
information is available for a terminated child process,
or a signal is received.
On return from a successful 
.I wait
call, 
the
.B status
area contains termination information about the process that exited
as defined below.
.PP
The
.I wait4
call provides a more general interface for programs
that need to wait for certain child processes,
that need resource utilization statistics accummulated by child processes,
or that require options.
The other wait functions are implemented using
.I wait4 .
.PP
The
.B wpid
parameter specifies the set of child processes for which to wait.
If
.B wpid
is -1, the call waits for any child process.
If
.B wpid
is 0,
the call waits for any child process in the process group of the caller.
If
.B wpid
is greater than zero, the call waits for the process with process id
.B wpid .
If
.B wpid
is less than -1, the call waits for any process whose process group id
equals the absolute value of
.B wpid .
.PP
The
.B status
parameter is defined below.  The
.B options
parameter contains the bitwise OR of any of the following options.
The
.B WNOHANG
option
is used to indicate that the call should not block if
there are no processes that wish to report status.
If the
.B WUNTRACED
option is set,
children of the current process that are stopped
due to a
.B SIGTTIN , SIGTTOU , SIGTSTP ,
or
.B SIGSTOP
signal also have
their status reported.
.PP
If
.B rusage
is non-zero, a summary of the resources used by the terminated
process and all its
children is returned (this information is currently not available
for stopped processes).
.PP
When the
.B WNOHANG
option is specified and no processes
wish to report status, 
.I wait4
returns a 
process id
of 0.
.PP
The
.I waitpid
call is identical to
.I wait4
with an
.B rusage
value of zero.
The older
.I wait3
call is the same as
.I wait4
with a
.B wpid
value of -1.
.PP
The following macros may be used to test the manner of exit of the process.
One of the first three macros will evaluate to a non-zero (true) value:
.sp
.I WIFEXITED(status)
\- True if the process terminated normally by a call to
.I _exit(2)
or
.I exit(2) .
.sp
.I WIFSIGNALED(status)
\- True if the process terminated due to receipt of a signal.
.sp
.I WIFSTOPPED(status)
\- True if the process has not terminated, but has stopped and can be restarted.
This macro can be true only if the wait call specified the
.B WUNTRACED
option
or if the child process is being traced (see
\fIptrace(2)\fP).
.PP
Depending on the values of those macros, the following macros
produce the remaining status information about the child process:
.sp
.I WEXITSTATUS(status)
\- If \fIWIFEXITED(status)\fP
is true, evaluates to the low-order 8 bits
of the argument passed to \fI_exit(2)\fP
or \fIexit(2)\fP
by the child.
.sp
.I WTERMSIG(status)
\- If \fIWIFSIGNALED(status)\fP
is true, evaluates to the number of the signal
that caused the termination of the process.
.sp
.I WCOREDUMP(status)
If \fIWIFSIGNALED(status)\fP
is true, evaluates as true if the termination
of the process was accompanied by the creation of a core file
containing an image of the process when the signal was received.
.sp
.I WSTOPSIG(status)\fP
If \fIWIFSTOPPED(status)\fP
is true, evaluates to the number of the signal
that caused the process to stop.
.SH NOTES
See
.I sigvec(2)
for a list of termination signals.
A status of 0 indicates normal termination.
.PP
If a parent process terminates without
waiting for all of its child processes to terminate,
the remaining child processes are assigned the parent
process 1 ID (the init process ID).
.PP
If a signal is caught while any of the
.I wait
calls is pending,
the call may be interrupted or restarted when the signal-catching routine
returns,
depending on the options in effect for the signal;
see \fIintro(2)\fP,
System call restart.
.SH RETURN VALUES
If
.I wait()
returns due to a stopped
or terminated child process, the process ID of the child
is returned to the calling process.  Otherwise, a value of -1
is returned and
.I errno
is set to indicate the error.
.PP
If
.I wait4(), wait3() or waitpid()
returns due to a stopped
or terminated child process, the process ID of the child
is returned to the calling process.
If there are no children not previously awaited,
-1 is returned with
.I errno
set to
.B [ECHILD].
Otherwise, if
.B WNOHANG
is specified and there are
no stopped or exited children,
0 is returned.
If an error is detected or a caught signal aborts the call,
a value of -1
is returned and
.I errno
is set to indicate the error.
.SH ERRORS
.I Wait()
will fail and return immediately if:
.TP 15
[ECHILD]
The calling process has no existing unwaited-for
child processes.
.TP 15
[EFAULT]
The \fIstatus\fP or \fIrusage\fP arguments point to an illegal address.
(May not be detected before exit of a child process.)
.TP 15
[EINTR]
The call was interrupted by a caught signal,
or the signal had the
.I SV_INTERRUPT
flag set.
.SH STANDARDS
The
.I wait
and
.I waitpid
functions are defined by POSIX;
.I wait4
and
.I wait3
are not specified by POSIX.
The
.I WCOREDUMP
macro
and the ability to restart a pending
.I wait
call are extensions to the POSIX interface.
.SH SEE ALSO
.I exit(2) ,
.I sigvec(2)
.Sh HISTORY
A
.I wait
function call appeared in Version 6 AT&T UNIX.
